with Apex_Address_To_String;
with Apex_Occurrence_Utils;
with Apex_Stack_Trace_Foundation;
with Apex_Stack_Trace_Io;
with V_I_Except;
with System;

package body Apex_Stack_Trace is

    pragma Suppress_All;

    Unhandled_Trace_Dump_Enabled : Boolean := False;
    Exception_Traceback_Started : Boolean := False;

    procedure Trace_Dump (Occurrence :
                             Apex_Occurrence_Types.Exception_Occurrence;
                          Filename : String := "trace_dump";
                          Append : Boolean := False;
                          Trace_Label : String := "") is
    begin
        Trace_Dump (Apex_Occurrence_Utils.To_Exception_Occurrence_Access
                       (Occurrence'Address), Filename, Append, Trace_Label);
    end Trace_Dump;


    procedure Trace_Dump (Occurrence :
                             Apex_Occurrence_Types.Exception_Occurrence_Access;
                          Filename : String := "trace_dump";
                          Append : Boolean := False;
                          Trace_Label : String := "") is
        Frame : Apex_Stack_Trace_Types.Frame;
        This_Oc : Apex_Occurrence_Types.Exception_Occurrence_Access :=
           Occurrence;
        Fd : Apex_Stack_Trace_Io.File_Descriptor;
    begin

        Fd := Apex_Stack_Trace_Io.Open (Filename, Append);

        while Apex_Occurrence_Types."/=" (This_Oc, null) loop
            Apex_Stack_Trace_Foundation.Initialize
               (User_Frame => Apex_Stack_Trace_Types.To_Frame_Access
                                 (Frame'Address),
                From_Occurrence => This_Oc);
            Apex_Stack_Trace_Io.Dump_Trace_Label
               (Fd, Apex_Occurrence_Utils.Exception_Name (This_Oc) &
                       Trace_Label);
            Apex_Stack_Trace_Io.Dump_Registers (Fd, This_Oc);
            Apex_Stack_Trace_Io.Dump_Stack_Trace
               (Fd, Apex_Stack_Trace_Types.To_Frame_Access (Frame'Address));
            This_Oc := Apex_Stack_Trace_Foundation.Next_Occurrence (This_Oc);
        end loop;

        Apex_Stack_Trace_Io.Close (Fd);

    end Trace_Dump;

    procedure Trace_Dump (Filename : String := "trace_dump";
                          Append : Boolean := False;
                          Trace_Label : String := "trace") is
        Frame : Apex_Stack_Trace_Types.Frame;
        Fd : Apex_Stack_Trace_Io.File_Descriptor;
    begin

        Fd := Apex_Stack_Trace_Io.Open (Filename);

        Apex_Stack_Trace_Foundation.Initialize
           (Apex_Stack_Trace_Types.To_Frame_Access (Frame'Address));
        Apex_Stack_Trace_Io.Dump_Trace_Label (Fd, Trace_Label);
        Apex_Stack_Trace_Io.Dump_Stack_Trace
           (Fd, Apex_Stack_Trace_Types.To_Frame_Access (Frame'Address));
        Apex_Stack_Trace_Io.Close (Fd);

    end Trace_Dump;

    procedure Trace_Dump (Context : Interrupt_Context_Access;
                          Filename : String := "trace_dump";
                          Append : Boolean := False;
                          Trace_Label : String := "interrupt_trace") is
        Frame : Apex_Stack_Trace_Types.Frame;
        Fd : Apex_Stack_Trace_Io.File_Descriptor;
    begin
        Fd := Apex_Stack_Trace_Io.Open (Filename);

        Apex_Stack_Trace_Foundation.Initialize
           (User_Frame => Apex_Stack_Trace_Types.To_Frame_Access
                             (Frame'Address),
            From_Context => Context);
        Apex_Stack_Trace_Io.Dump_Trace_Label (Fd, Trace_Label);
        Apex_Stack_Trace_Io.Dump_Stack_Trace
           (Fd, Apex_Stack_Trace_Types.To_Frame_Access (Frame'Address));
        Apex_Stack_Trace_Io.Close (Fd);
    end Trace_Dump;

    -- Cause a core dump if an unhandled exception occurs.  This is useful
    -- for doing post-mortem analysis via the debugger.  Note that we don't
    -- support core dumps on the cross.

    procedure Unhandled_Exception_Core_Dump_Enable is
    begin
        V_I_Except.Enable_Unhandled_Dump_Core;
    end Unhandled_Exception_Core_Dump_Enable;

    procedure Unhandled_Exception_Core_Dump_Disable is
    begin
        -- currently there's no way to turn off this feature.
        null; -- NYI
    end Unhandled_Exception_Core_Dump_Disable;

    -- Enable/Disable Unhandled Exception Stack Dumps 


    procedure Exception_Trace_Enable is
    begin
        V_I_Except.Enable_Traceback_Regs;
    end Exception_Trace_Enable;


end Apex_Stack_Trace;

