with Apex_Occurrence_Types;
with Apex_Stack_Trace_Types;
with System;
with Unchecked_Conversion;

package Apex_Stack_Trace_Foundation is

    use Apex_Stack_Trace_Types;

    procedure Initialize (User_Frame : Frame_Access);

    procedure Initialize (User_Frame : Frame_Access;
                          From_Context : Interrupt_Context_Access);

    pragma Interface (Ada, Initialize);
    pragma Import_Procedure (Internal => Initialize,
                             External => "__APEX_INIT_FRAME_FROM_CURRENT",
                             Parameter_Types => (Frame_Access));
    pragma Import_Procedure
              (Internal => Initialize,
               External => "__APEX_INIT_FRAME_FROM_INTERRUPT_CONTEXT",
               Parameter_Types => (Frame_Access,
                                   Interrupt_Context_Access));

    procedure Initialize (User_Frame : Frame_Access;
                          From_Occurrence : Apex_Occurrence_Types.
                                               Exception_Occurrence_Access);

    procedure Initialize (User_Frame : Frame_Access;
                          From_Occurrence : Apex_Occurrence_Types.
                                               Exception_Occurrence);
    pragma Suppress (Elaboration_Check, On => Initialize);

    -- This function returns a pointer to the current exception occurrence, iff
    -- Enable_Traceback_Regs has been called.
    --
    -- This function is only needed in Ada83 where the choice parameter
    -- syntax isn't available.

    function Exception_Occurrence_Current
                return Apex_Occurrence_Types.Exception_Occurrence_Access;
    pragma Interface (Ada, Exception_Occurrence_Current);
    pragma Import_Function (Internal => Exception_Occurrence_Current,
                            External => "__EXCEPTION_OCCURRENCE_CURRENT");


    function Stack_Pointer (User_Frame : Frame_Access) return System.Address;
    pragma Interface (Ada, Stack_Pointer);
    pragma Import_Function (Internal => Stack_Pointer,
                            External => "__STACK_POINTER_FROM_FRAME");

    function Frame_Pointer (User_Frame : Frame_Access) return System.Address;
    pragma Interface (Ada, Frame_Pointer);
    pragma Import_Function (Internal => Frame_Pointer,
                            External => "__FRAME_POINTER_FROM_FRAME");

    function Program_Counter (User_Frame : Frame_Access) return System.Address;
    pragma Interface (Ada, Program_Counter);
    pragma Import_Function (Internal => Program_Counter,
                            External => "__PROGRAM_COUNTER_FROM_FRAME");

    procedure Previous_Frame (User_Frame : Frame_Access;
                              Unwind_Complete : out Boolean);
    pragma Interface (Ada, Previous_Frame);
    pragma Import_Procedure (Internal => Previous_Frame,
                             External => "__PREVIOUS_FRAME");

    procedure Exception_Callout_Install
                 (Initial_Raise : System.Address := System.Null_Address;
                  Subsequent_Reraise : System.Address := System.Null_Address;
                  Unhandled : System.Address := System.Null_Address);

    pragma Interface (Ada, Exception_Callout_Install);

    pragma Import_Procedure
              (Internal => Exception_Callout_Install,
               External => "__EXCEPTION_CALLOUT_INSTALL_ADA83",
               Parameter_Types =>
                  (System.Address, System.Address, System.Address));

    function Next_Occurrence
                (Occurrence : Apex_Occurrence_Types.
                                 Exception_Occurrence_Access)
                return Apex_Occurrence_Types.Exception_Occurrence_Access;

    function Next_Occurrence (Occurrence :
                                 Apex_Occurrence_Types.Exception_Occurrence)
                             return Apex_Occurrence_Types.
                                    Exception_Occurrence_Access;
    pragma Suppress (Elaboration_Check, On => Next_Occurrence);

end Apex_Stack_Trace_Foundation;

