--
-- File Name: stack_trace_io.os_dependant.2.ada
--

with Unix_Operations;
with Unix_Types;
with Unix_Constants;

separate (Apex_Stack_Trace_Io)
package body Os_Dependent is

    pragma Suppress_All;

    function Open (Filename : String; Append : Boolean := False)
                  return File_Descriptor is
        use Unix_Types;
        Nt_Filename : constant String :=
           Filename & Ascii.Nul; -- null terminated
        Fd : File_Descriptor;
        Oflag : Unix_Types.Int := Unix_Constants.Unix_Fcntl.O_Creat +
                                     Unix_Constants.Unix_Fcntl.O_Wronly;
        Oflag2 : Unix_Types.Int;
    begin
        if Filename = "stdout" then
            -- Use the standard output file descriptor which is always a 1
            Fd := 1;
        elsif Filename = "stderr" then
            -- Use the standard error file descriptor which is always a 2
            Fd := 2;
        else
            if Append then
                Oflag2 := Oflag + Unix_Constants.Unix_Fcntl.O_Append;
            else
                Oflag2 := Oflag + Unix_Constants.Unix_Fcntl.O_Trunc;
            end if;
            Fd := Unix_Operations.Open
                     (Unix_Types.To_Char_Ptr
                         (Nt_Filename (Nt_Filename'First)'Address),
                      Oflag2, 8#666#);
            if Fd = -1 then
                Oflag2 := Oflag + Unix_Constants.Unix_Fcntl.O_Append;
                Fd := Unix_Operations.Open
                         (Unix_Types.To_Char_Ptr
                             (Nt_Filename (Nt_Filename'First)'Address),
                          Oflag2, 8#666#);
            end if;
            return Fd;
        end if;
    end Open;

    procedure Put_String (Fd : File_Descriptor; S : String) is
        use Unix_Types;
        Bytes_Written : Ssize_T;
    begin
        Bytes_Written :=
           Unix_Operations.Write (Fildes => Fd,
                                  Buf => To_Char_Ptr (S (S'First)'Address),
                                  Nbyte => S'Length);
    end Put_String;

    procedure Close (Fd : File_Descriptor) is
        Status : Unix_Types.Int;
        use Unix_Types;
    begin
        -- don't close stdout or stderr
        if Fd in 1 .. 2 then
            return;
        end if;
        Status := Unix_Operations.Close (Fd);
    end Close;


end Os_Dependent;

